      subroutine delifm(p,f,nlyrs,perta,pertb,pertr)
      integer nlyrs
      complex p,f
      real perta,pertb,pertr
c
c           compute kennett's interface matricies for layer perurbation
c        for a p, sv or sh wave incident, only for affected interfaces
c        for interface 0 at top of layer 1, a free surface, compute
c        reflection operator, and free surface displacement operator
c        layer n is half space
c        compute ru, rd, tu, td at interfaces
c        given frequency and phase slowness.
c
c          arguments...
c
c        f,p - prescribed freq (hz) & horizontal phase slowness (c is
c            not restricted to be greater than alfa or beta)
c            both may be complex
c
c        nlyrs - the number of  layers that are perturbed
c
c        perta - the perturbation of the compressional velocity
c
c        pertb - the perturbation of the shear velocity
c
c        pertr - the perturbation of the density
c
c
c        passed in common /model/
c        alfa,beta,qp,qs,rho and thik contain the medium properties for
c            layers 1 thru nlyrs (the halfspace)
c
c
c
c        commons and declarations
c
c
      include  'kennet.inc'
c
c        complex declarations
c
      complex mum11,mum12,mum21,mum22,mup11,mup12,mup21,mup22
      complex mdm11,mdm12,mdm21,mdm22,mdp11,mdp12,mdp21,mdp22
      complex num11,num12,num21,num22,nup11,nup12,nup21,nup22
      complex ndm11,ndm12,ndm21,ndm22,ndp11,ndp12,ndp21,ndp22
      complex xip,xim,etap,etam,epap,epam,epbp,epbm,mum,mup
      complex alfam,alfap,betam,betap,rhom,rhop
c
      complex i,one,two
      complex t1,t2,zshp,zshm
      complex t11,t12,t21,t22,det,l11,l12,l21,l22
      integer nif
      integer ipert,above,below
      parameter( above = 1, below = 2 )
      intrinsic csqrt
      complex vslow
      external vslow
      complex rhoprt,ap
      rhoprt(ap) = 0.77 + 0.32*ap
      data i/(0.,1.)/,one,two/(1.,0.),(2.,0.)/
c
c
c	perturb first layer to setup for loop
c
      alfam = alfa(1) * perta
      betam = beta(1) * pertb
c     rhom = rho(1) * pertr
      if ( pertr .ne. 1.0 ) then
	 ap = alfam
	 rhom = rhoprt(ap)
	else
	 rhom = rho(1)
      endif
      mum = betam*betam * rhom
      xim = vslow(alfam,p,f)
      etam = vslow(betam,p,f)
      dxi(1) = xim
      deta(1) =  etam
      epam = one / csqrt( two*rhom*xim )
      epbm = one / csqrt( two*rhom*etam )
      t1 = two * mum * p
      t2 = t1 * p - rhom
c
c        form layer matricies for perturbation in layer 1
c
      mdm11 = i * xim * epam
      mum11 = - mdm11
      mdm12 = p * epbm
      mum12 = mdm12
      mdm21 = p * epam
      mum21 = mdm21
      mdm22 = i * etam * epbm
      mum22 = - mdm22
      ndm11 = t2 * epam
      num11 = ndm11
      ndm12 = t1 * mdm22
      num12 = -ndm12
      ndm21 = t1 * mdm11
      num21 = -ndm21
      ndm22 = t2 * epbm
      num22 = ndm22
      zshm = mum * etam
c
c	now setup, so fall into loop
c
      do 10 nif = 1, nlyrs-1
c
c     copy the n and m matrices if this is source layer
c
         if ( nif .eq. srclyr ) then
            dxis = xim
            detas = etam
            depas = epam
            depbs = epbm
            dnus11 = num11
            dnus12 = num12
            dnus21 = num21
            dnus22 = num22
            dnussh = -i*rhom*betam*etam*epbm
            dnds11 = ndm11
            dnds12 = ndm12
            dnds21 = ndm21
            dnds22 = ndm22
            dndssh = -dnussh
            dmus11 = mum11
            dmus12 = mum12
            dmus21 = mum21
            dmus22 = mum22
            dmussh = epbm/betam
            dmds11 = mdm11
            dmds12 = mdm12
            dmds21 = mdm21
            dmds22 = mdm22
            dmdssh = dmussh
            drhos = rhom
            dalfas = alfam
            dbetas = betam
          endif
c
       do 20 ipert = above, below
c
c	compute perturbation layer above or below the interface
c	and get all layer matrices before going into the
c	computation of interface matrices
c
	if ( ipert .eq. above ) then
         nup11 = nu11(nif+1)
         nup12 = nu12(nif+1)
         nup21 = nu21(nif+1)
         nup22 = nu22(nif+1)
         ndp11 = nd11(nif+1)
         ndp12 = nd12(nif+1)
         ndp21 = nd21(nif+1)
         ndp22 = nd22(nif+1)
         mup11 = mu11(nif+1)
         mup12 = mu12(nif+1)
         mup21 = mu21(nif+1)
         mup22 = mu22(nif+1)
         mdp11 = md11(nif+1)
         mdp12 = md12(nif+1)
         mdp21 = md21(nif+1)
         mdp22 = md22(nif+1)
         zshp = zsh(nif+1)
        else if ( ipert .eq. below ) then
         alfap = alfa(nif+1) * perta
         betap = beta(nif+1) * pertb
c        rhop = rho(nif+1) * pertr
	 if ( pertr .ne. 1.0 ) then
	    ap = alfap
	    rhop = rhoprt(ap)
	   else
	    rhop = rho(nif+1)
	 endif
         mup = betap*betap * rhop
         xip = vslow(alfap,p,f)
         etap = vslow(betap,p,f)
         dxi(nif+1) = xip
         deta(nif+1) =  etap
         epap = one / csqrt( two*rhop*xip )
         epbp = one / csqrt( two*rhop*etap )
         t1 = two * mup * p
         t2 = t1 * p - rhop
c
c        form layer matricies for perurbation below
c
         mdp11 = i * xip * epap
         mup11 = - mdp11
         mdp12 = p * epbp
         mup12 = mdp12
         mdp21 = p * epap
         mup21 = mdp21
         mdp22 = i * etap * epbp
         mup22 = - mdp22
         ndp11 = t2 * epap
         nup11 = ndp11
         ndp12 = t1 * mdp22
         nup12 = -ndp12
         ndp21 = t1 * mdp11
         nup21 = -ndp21
         ndp22 = t2 * epbp
         nup22 = ndp22
         zshp = mup * etap
c
c	retrieve the saved (unperturbed) matrices for layer
c	above the interface, perturbation is below
c
         num11 = nu11(nif)
         num12 = nu12(nif)
         num21 = nu21(nif)
         num22 = nu22(nif)
         ndm11 = nd11(nif)
         ndm12 = nd12(nif)
         ndm21 = nd21(nif)
         ndm22 = nd22(nif)
         mum11 = mu11(nif)
         mum12 = mu12(nif)
         mum21 = mu21(nif)
         mum22 = mu22(nif)
         mdm11 = md11(nif)
         mdm12 = md12(nif)
         mdm21 = md21(nif)
         mdm22 = md22(nif)
         zshm = zsh(nif)
	endif
c
c	now the perturbed layer matrices above interface
c	and unperturbed layer matrices below interface
c	are ready, begin interface loop with two sets of
c	interface matrices for each interface,
c	one set perturbed above the interface and
c	one set perturbed below the interface
c
c
        if ( ( nif .eq. 1 ) .and. ( ipert .eq. above ) ) then
c
c        calculate the free surface reflection matrix, and free surface
c        free surface displacement operator.
c
         det = ndm11*ndm22 - ndm12*ndm21
         det = one/det
         t11 = -ndm22*det
         t22 = -ndm11*det
         t12 = ndm12*det
         t21 = ndm21*det
         druppfs = t11*num11 + t12*num21
         drupsfs = t11*num12 + t12*num22
         druspfs = t21*num11 + t22*num21
         drussfs = t21*num12 + t22*num22
         drushfs = one
         ddvpfs = mum11 + mdm11*druppfs + mdm12*druspfs
         ddrpfs = mum21 + mdm21*druppfs + mdm22*druspfs
         ddvsfs = mum12 + mdm11*drupsfs + mdm12*drussfs
         ddrsfs = mum22 + mdm21*drupsfs + mdm22*drussfs
         ddtshfs = two * epbm / betam
	endif
c
c	now do the interface matrices 
c
         t11 = mum11*ndp11 + mum21*ndp21 - num11*mdp11 - num21*mdp21
         t21 = mum12*ndp11 + mum22*ndp21 - num12*mdp11 - num22*mdp21
         t12 = mum11*ndp12 + mum21*ndp22 - num11*mdp12 - num21*mdp22
         t22 = mum12*ndp12 + mum22*ndp22 - num12*mdp12 - num22*mdp22
         det = t11*t22 - t12*t21
	 det = one/det
         l12 = -t12*det
         l21 = -t21*det
         l22 = t11*det
         l11 = t22*det
c
c
	 if ( ipert .eq. above ) then
            datdpp(nif) = i*l11
            datdps(nif) = i*l12
            datdsp(nif) = i*l21
            datdss(nif) = i*l22
            datupp(nif) = i*l11
            datups(nif) = i*l21
            datusp(nif) = i*l12
            datuss(nif) = i*l22
            datush(nif) = two*csqrt(zshp*zshm)/(zshp+zshm)
            datdsh(nif) = datush(nif)
	   else if ( ipert .eq. below ) then
            dbtdpp(nif) = i*l11
            dbtdps(nif) = i*l12
            dbtdsp(nif) = i*l21
            dbtdss(nif) = i*l22
            dbtupp(nif) = i*l11
            dbtups(nif) = i*l21
            dbtusp(nif) = i*l12
            dbtuss(nif) = i*l22
            dbtush(nif) = two*csqrt(zshp*zshm)/(zshp+zshm)
            dbtdsh(nif) = dbtush(nif)
	  endif
c
         t11 = mdm11*ndp11 + mdm21*ndp21 - ndm11*mdp11 - ndm21*mdp21
         t21 = mdm12*ndp11 + mdm22*ndp21 - ndm12*mdp11 - ndm22*mdp21
         t12 = mdm11*ndp12 + mdm21*ndp22 - ndm11*mdp12 - ndm21*mdp22
         t22 = mdm12*ndp12 + mdm22*ndp22 - ndm12*mdp12 - ndm22*mdp22
	 if ( ipert .eq. above ) then
            dardpp(nif) = - t11*l11 - t12*l21
            dardps(nif) = - t11*l12 - t12*l22
            dardsp(nif) = - t21*l11 - t22*l21
            dardss(nif) = - t21*l12 - t22*l22
            dardsh(nif) = (zshm - zshp)/(zshm + zshp)
	   else if ( ipert .eq. below ) then
            dbrdpp(nif) = - t11*l11 - t12*l21
            dbrdps(nif) = - t11*l12 - t12*l22
            dbrdsp(nif) = - t21*l11 - t22*l21
            dbrdss(nif) = - t21*l12 - t22*l22
            dbrdsh(nif) = (zshm - zshp)/(zshm + zshp)
	  endif
c
         t11 = mum11*nup11 + mum21*nup21 - num11*mup11 - num21*mup21
         t21 = mum12*nup11 + mum22*nup21 - num12*mup11 - num22*mup21
         t12 = mum11*nup12 + mum21*nup22 - num11*mup12 - num21*mup22
         t22 = mum12*nup12 + mum22*nup22 - num12*mup12 - num22*mup22
	 if ( ipert .eq. above ) then
            darupp(nif) = - l11*t11 - l12*t21
            darups(nif) = - l11*t12 - l12*t22
            darusp(nif) = - l21*t11 - l22*t21
            daruss(nif) = - l21*t12 - l22*t22
            darush(nif) = - dardsh(nif)
	   else if ( ipert .eq. below ) then
            dbrupp(nif) = - l11*t11 - l12*t21
            dbrups(nif) = - l11*t12 - l12*t22
            dbrusp(nif) = - l21*t11 - l22*t21
            dbruss(nif) = - l21*t12 - l22*t22
            dbrush(nif) = - dbrdsh(nif)
	  endif
c
c	 copy the above values to storage for  inversion
c        copy the below values to above values for next interface
c
      if ( ipert .eq. below ) then
         alfam = alfap
         betam = betap
         rhom = rhop
         mum = mup
         xim = xip
         etam = etap
         epam = epap
         epbm = epbp
         num11 = nup11
         num12 = nup12
         num21 = nup21
         num22 = nup22
         ndm11 = ndp11
         ndm12 = ndp12
         ndm21 = ndp21
         ndm22 = ndp22
         mum11 = mup11
         mum12 = mup12
         mum21 = mup21
         mum22 = mup22
         mdm11 = mdp11
         mdm12 = mdp12
         mdm21 = mdp21
         mdm22 = mdp22
	 zshm = zshp
      endif
20    continue
10    continue
c
c
c
      return
      end
