      subroutine delrcvru(p,f,plyr,nlyrs,dvp,dvs,drp,drs,dts,rpp,rps,rsp,rss,rsh)
      integer plyr,nlyrs
      complex p,f
      complex dvp,dvs,drp,drs,dts
      complex rpp,rps,rsp,rss,rsh
c
c                                       fS
c        compute receiver function and Ru (reflectivity bounded by free surf.)
c        receiver function - free surface displacement from a
c        plane wave incident from below, on a stack of plane, parallel,
c        homogeneous layers using the perturbed interface matricies
c	 and the perturbed layer parameters
c        for a p, sv or sh wave incident
c        interface 0 is top of layer 1, a free surface,
c        layer n is half space
c        given frequency and phase slowness.
c        plyr (perturbed) can be any of [1,nlyrs]
c
c        assumes that rcvr-rufs top-down and bottom-up implementatios have already
c        been called and saved the partial structural response terms used here
c
c          arguments...
c
c        f,p - prescribed freq (hz) & horizontal phase slowness (c is
c            not restricted to be greater than alfa or beta)
c            both may be complex
c
c        passed in common /model/
c        alfa,beta,qp,qs,rho and thik contain the medium properties for
c
c        passed in the commons /invsav?/
c        d[ab][tr][ud][ps][psh] - perturbed refl. and trans. interface matrices
c        s[rt]n[ud][ps][psh] - saved partial structure results from top-dn and bot-up
c
c        nlyrs - total number of layers, layer nlyrs is
c            the half space
c
c	 plyr - the perturbed layer can be any of [1,nlyrs]
c
c
c        commons and declarations
c
c
      include 'kennet.inc'
c
c        complex declarations
c
      complex i,zero,one,two,w
      complex t11,t12,t21,t22,l11,l12,l21,l22,tsh,lsh
      complex r11,r12,r21,r22,s11,s12,s21,s22,ssh
c     complex*16 det
      complex det
      complex x11,x12,x21,x22,y11,y12,y21,y22,xsh,ysh
      complex tnupp,tnups,tnusp,tnuss,tnush
      complex rndpp,rndps,rndsp,rndss,rndsh
      complex rnaupp,rnaups,rnausp,rnauss,rnaush
      complex rnbupp,rnbups,rnbusp,rnbuss,rnbush
      complex tdvp,tdvs,tdrp,tdrs,tdts
      complex phtp,phts,phtpp,phtps,phtss
      real twopi
      integer lyr,nif,cnvnif
      external cphs
      complex cphs
      data i,zero/(0.,1.),(0.,0.)/,one,two/(1.,0.),(2.,0.)/
      data twopi/6.2831853064/
c
c
c     Break computation down into 3 step
c     1.  The region above plyr
c     2.  The region below plyr
c     3.  Join the results from steps 1 and 2 into net RcvrFn and RufS
c
c
c     handle the special case of a half space
c
      w = twopi * f
      if ( nlyrs .eq. 1 ) then
         dvp = ddvpfs
         dvs = ddvsfs
         drp = ddrpfs
         drs = ddrsfs
         dts = ddtshfs
	 rpp = druppfs
	 rps = drupsfs
	 rsp = druspfs
	 rss = drussfs
	 rsh = drushfs
         return
       endif
c
c================================================================================
c
c   1. Region above plyr
c        Add perturbed interface (TopDown) or if plyr == 1  only perturbed FreeSurf
c
      if ( plyr .eq. 1 ) then
        lyr = 1
        if ( cnvnif .eq. allphs ) then
         rnaupp = druppfs
         rnauss = drussfs
         rnaups = drupsfs
         rnausp = druspfs
         rnaush = drushfs
        else if ( cnvnif .eq. prmphs ) then
         rnaupp = druppfs
         rnauss = drussfs
         rnaups = zero
         rnausp = zero
         rnaush = drushfs
        else if ( cnvnif .eq. cnvphs ) then
         rnaups = drupsfs
         rnausp = druspfs
         rnaupp = zero
         rnauss = zero
         rnaush = drushfs
        endif
c
c       initialize the displacement for reciever function
c
         tdvp = ddvpfs
         tdrp = ddrpfs
         tdvs = ddvsfs
         tdrs = ddrsfs
         tdts = ddtshfs
c
c       add the interface which is the top of the perturbed layer
c
       else
        rnaupp = srnupp_abtd(plyr-1)
        rnaups = srnups_abtd(plyr-1)
        rnausp = srnusp_abtd(plyr-1)
        rnauss = srnuss_abtd(plyr-1)
        rnaush = srnush_abtd(plyr-1)
        tdvp = sdvp(plyr-1)
        tdrp = sdrp(plyr-1)
        tdvs = sdvs(plyr-1)
        tdrs = sdrs(plyr-1)
        tdts = sdts(plyr-1)
c
c        form the reverberation operator for the layer above plyr using TopDown
c
         nif = plyr - 1
         cnvnif = cnvrsn(nif)
         if ( cnvnif .eq. allphs ) then
            t11 = dbrdpp(nif)
            t22 = dbrdss(nif)
            t12 = dbrdps(nif)
            t21 = dbrdsp(nif)
            tsh = dbrdsh(nif)
          else if ( cnvnif .eq. prmphs ) then
            t11 = dbrdpp(nif)
            t22 = dbrdss(nif)
            t12 = zero
            t21 = zero
            tsh = dbrdsh(nif)
          else if ( cnvnif .eq. cnvphs ) then
            t12 = dbrdps(nif)
            t21 = dbrdsp(nif)
            t11 = zero
            t22 = zero
            tsh = dbrdsh(nif)
          endif
         if ( reverb(lyr) .eq. allrvb ) then
            l11 = one - (rnaupp*t11 + rnaups*t21)
            l22 = one - (rnausp*t12 + rnauss*t22)
            l12 = - (rnaupp*t12 + rnaups*t22)
            l21 = - (rnausp*t11 + rnauss*t21)
            det = ( l11*l22 - l12*l21 )
            det = one / det
            l12 = -l12*det
            l21 = -l21*det
            t11 = l11*det
            l11 = l22*det
            l22 = t11
            lsh = one / ( one - rnaush*tsh )
         else if ( reverb(lyr) .eq. onervb ) then
            l11 = one + (rnaupp*t11 + rnaups*t21)
            l22 = one + (rnausp*t12 + rnauss*t22)
            l12 =  (rnaupp*t12 + rnaups*t22)
            l21 =  (rnausp*t11 + rnauss*t21)
            lsh = one + rnaush*tsh
         else if ( reverb(lyr) .eq. norvb ) then
            l11 = one
            l22 = one
            l12 = zero
            l21 = zero
            lsh = one
          endif
c
c        now finish the recursion, adding the next interface at top of plyr
c
         if ( cnvnif .eq. allphs ) then
            x11 = dbtdpp(nif)
            x22 = dbtdss(nif)
            x12 = dbtdps(nif)
            x21 = dbtdsp(nif)
            xsh = dbtdsh(nif)
            y11 = dbrupp(nif)
            y22 = dbruss(nif)
            y12 = dbrups(nif)
            y21 = dbrusp(nif)
            ysh = dbrush(nif)
          else if ( cnvnif .eq. prmphs ) then
            x11 = dbtdpp(nif)
            x22 = dbtdss(nif)
            x12 = zero
            x21 = zero
            xsh = dbtdsh(nif)
            y11 = dbrupp(nif)
            y22 = dbruss(nif)
            y12 = zero
            y21 = zero
            ysh = dbrush(nif)
          else if ( cnvnif .eq. cnvphs ) then
            x12 = dbtdps(nif)
            x21 = dbtdsp(nif)
            x11 = zero
            x22 = zero
            xsh = dbtdsh(nif)
            y12 = dbrups(nif)
            y21 = dbrusp(nif)
            y11 = zero
            y22 = zero
            ysh = dbrush(nif)
          endif
c
c    Using the transpose of the Reverberation operator computed above
c    which reverses the order of the 2 reflection matrices in the
c    reverberation.  Also using the transpose relation for Tu and Td
c
c        t11 = l11*tupp(nif) + l21*tusp(nif)
c        t12 = l11*tups(nif) + l21*tuss(nif)
c        t21 = l12*tupp(nif) + l22*tusp(nif)
c        t22 = l12*tups(nif) + l22*tuss(nif)
         t11 = l11*x11 + l21*x12
         t12 = l11*x21 + l21*x22
         t21 = l12*x11 + l22*x12
         t22 = l12*x21 + l22*x22
         tsh = lsh * xsh
         l11 = rnaupp*t11 + rnaups*t21
         l12 = rnaupp*t12 + rnaups*t22
         l21 = rnausp*t11 + rnauss*t21
         l22 = rnausp*t12 + rnauss*t22
         lsh = rnaush * tsh
c        rnupp = rupp(nif) + tdpp(nif)*l11 + tdps(nif)*l21
c        rnuss = russ(nif) + tdsp(nif)*l12 + tdss(nif)*l22
c        rnups = rups(nif) + tdpp(nif)*l12 + tdps(nif)*l22
c        rnusp = rusp(nif) + tdsp(nif)*l11 + tdss(nif)*l21
         rnaupp = y11 + x11*l11 + x12*l21
         rnauss = y22 + x21*l12 + x22*l22
         rnaups = y12 + x11*l12 + x12*l22
         rnausp = y21 + x21*l11 + x22*l21
         rnaush = ysh + xsh*lsh
         x11 = tdvp*t11 + tdvs*t21
         x12 = tdvp*t12 + tdvs*t22
         x21 = tdrp*t11 + tdrs*t21
         x22 = tdrp*t12 + tdrs*t22
         tdvp = x11
         tdvs = x12
         tdrp = x21
         tdrs = x22
         tdts = tdts*tsh
      endif
c
c   If plyr == nylrs  the added interface finishes the computation!!
c
      if ( plyr .eq. nlyrs ) then
	 dvp = tdvp
	 drp = tdrp
	 dvs = tdvs
	 drs = tdrs
	 dts = tdts
	 rpp = rnaupp
	 rps = rnaups
	 rsp = rnausp
	 rss = rnauss
	 rsh = rnaush
	 return
      endif
c
c     Now structure is complete from Free Surface down to top of Perturbed Layer (plyr)
c
c============================================================================================
c
c  2. Now compute response for Region below Perturbed Layer (plyr)
c       now add perutbed interface (BotUp) to the region Below plyr
c       (if plyr = nylr only perturbed lowest interface and part 1 took care of it)
c
      if ( plyr .eq. nlyrs-1 ) then
	 tnupp = datupp(nlyrs - 1)
	 tnups = datups(nlyrs - 1)
	 tnusp = datusp(nlyrs - 1)
	 tnuss = datuss(nlyrs - 1)
	 tnush = datush(nlyrs - 1)
	 rndpp = dardpp(nlyrs - 1)
	 rndps = dardps(nlyrs - 1)
	 rndsp = dardsp(nlyrs - 1)
	 rndss = dardss(nlyrs - 1)
	 rndsh = dardsh(nlyrs - 1)
	 rnbupp = darupp(nlyrs - 1)
	 rnbups = darups(nlyrs - 1)
	 rnbusp = darusp(nlyrs - 1)
	 rnbuss = daruss(nlyrs - 1)
	 rnbush = darush(nlyrs - 1)
      else
      lyr = plyr + 1
      tnupp  =  stnupp_ab(lyr)
      tnups  =  stnups_ab(lyr)
      tnusp  =  stnusp_ab(lyr)
      tnuss  =  stnuss_ab(lyr)
      tnush  =  stnush_ab(lyr)
      rndpp  =  srndpp_ab(lyr)
      rndps  =  srndps_ab(lyr)
      rndsp  =  srndsp_ab(lyr)
      rndss  =  srndss_ab(lyr)
      rndsh  =  srndsh_ab(lyr)
      rnbupp  =  srnupp_abbu(lyr)
      rnbups  =  srnups_abbu(lyr)
      rnbusp  =  srnusp_abbu(lyr)
      rnbuss  =  srnuss_abbu(lyr)
      rnbush  =  srnush_abbu(lyr)
      nif = plyr
      cnvnif = cnvrsn(nif)
      if ( cnvnif .eq. allphs ) then
         r11 = darupp(nif)
         r22 = daruss(nif)
         r12 = darups(nif)
         r21 = darusp(nif)
         rsh = darush(nif)
       else if ( cnvnif .eq. prmphs ) then
         r11 = darupp(nif)
         r22 = daruss(nif)
         r12 = zero
         r21 = zero
         rsh = darush(nif)
       else if ( cnvnif .eq. cnvphs ) then
         r12 = darups(nif)
         r21 = darusp(nif)
         r11 = zero
         r22 = zero
         rsh = darush(nif)
      endif
c
c          form (I - Rd(plyr+1)*Ru(plyr)]**-1 for upward tranmission
c          and reflection matrices Tu, Ru (6.4, 6.18)
c
      if ( reverb(lyr) .eq. allrvb ) then
         l11 = one - (rndpp*r11 + rndps*r21)
         l22 = one - (rndsp*r12 + rndss*r22)
         l12 = - (rndpp*r12 + rndps*r22)
         l21 = - (rndsp*r11 + rndss*r21)
         det = ( l11*l22 - l12*l21 )
c            find inverse
         det = one / det
         l12 = -l12*det
         l21 = -l21*det
         t11 = l11*det
         l11 = l22*det
         l22 = t11
         lsh = one / ( one - rndsh*rsh )
       else if ( reverb(lyr) .eq. onervb ) then
         l11 = one + (rndpp*r11 + rndps*r21)
         l22 = one + (rndsp*r12 + rndss*r22)
         l12 =  (rndpp*r12 + rndps*r22)
         l21 =  (rndsp*r11 + rndss*r21)
         lsh = one + rndsh*rsh
       else if ( reverb(lyr) .eq. norvb ) then
         l11 = one
         l22 = one
         l12 = zero
         l21 = zero
         lsh = one
      endif
c
c        now finish the recursion, adding the next interface
c
      if ( cnvnif .eq. allphs ) then
         x11 = datupp(nif)
         x22 = datuss(nif)
         x12 = datups(nif)
         x21 = datusp(nif)
         xsh = datush(nif)
         y11 = dardpp(nif)
         y22 = dardss(nif)
         y12 = dardps(nif)
         y21 = dardsp(nif)
         ysh = dardsh(nif)
       else if ( cnvnif .eq. prmphs ) then
         x11 = datupp(nif)
         x22 = datuss(nif)
         x12 = zero
         x21 = zero
         xsh = datush(nif)
         y11 = dardpp(nif)
         y22 = dardss(nif)
         y12 = zero
         y21 = zero
         ysh = dardsh(nif)
       else if ( cnvnif .eq. cnvphs ) then
         x12 = datups(nif)
         x21 = datusp(nif)
         x11 = zero
         x22 = zero
         xsh = datush(nif)
         y12 = dardps(nif)
         y21 = dardsp(nif)
         y11 = zero
         y22 = zero
         ysh = dardsh(nif)
      endif
c
c         post multiply reverberation operator for Tu (6.4)
c
      t11 = l11*tnupp + l12*tnusp
      t22 = l21*tnups + l22*tnuss
      t21 = l21*tnupp + l22*tnusp
      t12 = l11*tnups + l12*tnuss
      tsh = lsh * tnush
c
c                               -1                    T
c        Ru = Ru + TdRu[I - RdRu] Tu  ; but use Td = Tu  relation
c
      s11 = r11*t11 + r12*t21
      s12 = r11*t12 + r12*t22
      s21 = r21*t11 + r22*t21
      s22 = r21*t12 + r22*t22
      ssh = rsh*tsh
      rnbupp = rnbupp + tnupp*s11 + tnusp*s21
      rnbups = rnbups + tnupp*s12 + tnusp*s22
      rnbusp = rnbusp + tnups*s11 + tnuss*s21
      rnbuss = rnbuss + tnups*s12 + tnuss*s22
      rnbush = rnbush + tnush*ssh
c
c        tnupp = tupp(nif)*t11 + tups(nif)*t21
c        tnuss = tusp(nif)*t12 + tuss(nif)*t22
c        tnups = tupp(nif)*t12 + tups(nif)*t22
c        tnusp = tusp(nif)*t11 + tuss(nif)*t21
c         pre multiply for Tu (6.4, 6.18)
c         Tu(z-) = Tu(AB)[I - Rd(BC)Ru(AB)]**-1 Tu(BC)
c
      tnupp = x11*t11 + x12*t21
      tnuss = x21*t12 + x22*t22
      tnups = x11*t12 + x12*t22
      tnusp = x21*t11 + x22*t21
      tnush = xsh * tsh
c
c     Now use the transpose of the reverberation operator above to reverse
c     the order of the reflection operators to form recursion Rd
c     previous was for Tu and Ru
c
c        t11 = l11*tdpp(nif) + l21*tdsp(nif)
c        t12 = l11*tdps(nif) + l21*tdss(nif)
c        t21 = l12*tdpp(nif) + l22*tdsp(nif)
c        t22 = l12*tdps(nif) + l22*tdss(nif)
      t11 = l11*x11 + l21*x12
      t12 = l11*x21 + l21*x22
      t21 = l12*x11 + l22*x12
      t22 = l12*x21 + l22*x22
      tsh = lsh * xsh
      l11 = rndpp*t11 + rndps*t21
      l12 = rndpp*t12 + rndps*t22
      l21 = rndsp*t11 + rndss*t21
      l22 = rndsp*t12 + rndss*t22
      lsh = rndsh * tsh
c        rndpp = rdpp(nif) + tupp(nif)*l11 + tups(nif)*l21
c        rndss = rdss(nif) + tusp(nif)*l12 + tuss(nif)*l22
c        rndps = rdps(nif) + tupp(nif)*l12 + tups(nif)*l22
c        rndsp = rdsp(nif) + tusp(nif)*l11 + tuss(nif)*l21
c         Rd(AC) (6.3)
      rndpp = y11 + x11*l11 + x12*l21
      rndss = y22 + x21*l12 + x22*l22
      rndps = y12 + x11*l12 + x12*l22
      rndsp = y21 + x21*l11 + x22*l21
      rndsh = ysh + xsh*lsh
      endif
c
c================================================================================
c
c     3. Now Join the Results from above plyr and below plyr with the
c          Revererations in plyr using the phase from pertrubed vertical slowness.
c
      lyr = plyr
      phtp = cphs( -i*w*dxi(plyr)*thik(plyr) )
      phts = cphs( -i*w*deta(plyr)*thik(plyr) )
      phtpp = phtp * phtp
      phtps = phtp * phts
      phtss = phts * phts
      tnupp = tnupp * phtp
      tnuss = tnuss * phts
      tnups = tnups * phtp
      tnusp = tnusp * phts
      tnush = tnush * phts
      rndpp = rndpp * phtpp
      rndss = rndss * phtss
      rndps = rndps * phtps
      rndsp = rndsp * phtps
      rndsh = rndsh * phtss
c
c     Now form the Recursion in plyr using above and below reults
c
      if ( reverb(lyr) .eq. allrvb ) then
         l11 = one - (rndpp*rnaupp + rndps*rnausp)
         l22 = one - (rndsp*rnaups + rndss*rnauss)
         l12 = - (rndpp*rnaups + rndps*rnauss)
         l21 = - (rndsp*rnaupp + rndss*rnausp)
         det = ( l11*l22 - l12*l21 )
         det = one / det
         l12 = -l12*det
         l21 = -l21*det
         t11 = l11*det
         l11 = l22*det
         l22 = t11
         lsh = one / ( one - rndsh*rnaush )
       else if ( reverb(lyr) .eq. onervb ) then
         l11 = one + (rndpp*rnaupp + rndps*rnausp)
         l22 = one + (rndsp*rnaups + rndss*rnauss)
         l12 =  (rndpp*rnaups + rndps*rnauss)
         l21 =  (rndsp*rnaupp + rndss*rnausp)
         lsh = one + rndsh*rnaush
       else if ( reverb(lyr) .eq. norvb ) then
         l11 = one
         l22 = one
         l12 = zero
         l21 = zero
         lsh = one
      endif
c
c     Form the Rcvr Fn and RufS with the reverberation and pieces
c
      t11 = l11*tnupp + l12*tnusp
      t22 = l22*tnuss + l21*tnups
      t12 = l11*tnups + l12*tnuss
      t21 = l22*tnusp + l21*tnupp
      tsh = lsh*tnush
c
      dvp = tdvp*t11 + tdvs*t21
      dvs = tdvs*t22 + tdvp*t12
      drp = tdrp*t11 + tdrs*t21
      drs = tdrs*t22 + tdrp*t12
      dts = tdts*tsh
c
      s11 = rnaupp*t11 + rnaups*t21
      s22 = rnauss*t22 + rnausp*t12
      s12 = rnaupp*t12 + rnaups*t22
      s21 = rnausp*t11 + rnauss*t21
      ssh = rnaush*tsh
c
      rpp = rnbupp + tnupp*s11 + tnusp*s21
      rss = rnbuss + tnuss*s22 + tnups*s12
      rps = rnbups + tnusp*s22 + tnupp*s12
      rsp = rnbusp + tnups*s11 + tnuss*s21
      rsh = rnbush + tnush*ssh
c
c
c
      return
      end
