      subroutine delrefl(p,f,plyr,tplyr,btlyr,rpp,rps,rsp,rss,rsh)
      integer plyr,tplyr,btlyr
      complex p,f
      complex rpp,rps,rsp,rss,rsh
c
c
c        compute Rd (reflectivity bounded above by source lyr, below by half space)
c        plane wave incident from above, on a stack of plane, parallel,
c        homogeneous layers using the perturbed interface matricies
c	 and the perturbed layer parameters
c        for a p, sv or sh wave incident
c        tplyr is just the source layer
c        btlyr or layer n is half space
c        given frequency and phase slowness.
c        plyr (perturbed) can be any of [tplyr,btlyr]
c
c        assumes that refl top-down and bottom-up implementatios have already
c        been called and saved the partial structural response terms used here
c
c          arguments...
c
c        f,p - prescribed freq (hz) & horizontal phase slowness (c is
c            not restricted to be greater than alfa or beta)
c            both may be complex
c
c        passed in common /model/
c        alfa,beta,qp,qs,rho and thik contain the medium properties for
c
c        passed in the commons /invsav?/
c        d[ab][tr][ud][ps][psh] - perturbed refl. and trans. interface matrices
c        s[rt]n[ud][ps][psh] - saved partial structure results from top-dn and bot-up
c
c        tplyr,btlyr - top layer is the src layer [tplyr] and bottom layer [btlyr]
c            the half space
c
c	 plyr - the perturbed layer can be any of [tplyr,btlyr]
c
c
c        commons and declarations
c
c
      include 'kennet.inc'
c
c        complex declarations
c
      complex i,zero,one,two,w
      complex t11,t12,t21,t22,l11,l12,l21,l22,tsh,lsh
      complex r11,r12,r21,r22,s11,s12,s21,s22,ssh
c     complex*16 det
      complex det
      complex x11,x12,x21,x22,y11,y12,y21,y22,xsh,ysh
      complex tndpp,tndps,tndsp,tndss,tndsh
      complex rnaupp,rnaups,rnausp,rnauss,rnaush
      complex rndpp,rndps,rndsp,rndss,rndsh
      complex rnadpp,rnadps,rnadsp,rnadss,rnadsh
      complex rnbdpp,rnbdps,rnbdsp,rnbdss,rnbdsh
      complex phtp,phts,phtpp,phtps,phtss
      real twopi
      integer lyr,nif,cnvnif
      external cphs
      complex cphs
      data i,zero/(0.,1.),(0.,0.)/,one,two/(1.,0.),(2.,0.)/
      data twopi/6.2831853064/
c
c
c     Break computation down into 3 step
c     1.  The region above plyr
c     2.  The region below plyr
c     3.  Join the results from steps 1 and 2 into net Rd
c
c
c
      w = twopi * f
c
c================================================================================
c
c   1. Region above plyr
c        If plyr > tplyr
c        Add perturbed interface (TopDown) which is top of plyr
c        if plyr == tplyr  : No action, Bottom Up adds the interface
c        if plyr == tplyr + 1  : Just get the petrubed interface
c        if blyr > plyr > tplyr + 1  : add perturbed interface
c        if plyr == blyr  : add bottom perturbed and return
c
      if ( plyr .gt. tplyr ) then
	 if ( plyr .eq. tplyr + 1 ) then
	    tndpp = dbtdpp(plyr - 1)
	    tndps = dbtdps(plyr - 1)
	    tndsp = dbtdsp(plyr - 1)
	    tndss = dbtdss(plyr - 1)
	    tndsh = dbtdsh(plyr - 1)
	    rnadpp = dbrdpp(plyr - 1)
	    rnadps = dbrdps(plyr - 1)
	    rnadsp = dbrdsp(plyr - 1)
	    rnadss = dbrdss(plyr - 1)
	    rnadsh = dbrdsh(plyr - 1)
	    rnaupp = dbrupp(plyr - 1)
	    rnaups = dbrups(plyr - 1)
	    rnausp = dbrusp(plyr - 1)
	    rnauss = dbruss(plyr - 1)
	    rnaush = dbrush(plyr - 1)
	  else
            lyr = plyr - 1
	    nif = plyr - 1
            cnvnif = cnvrsn(nif)
            if ( cnvnif .eq. allphs ) then
               r11 = dbrdpp(nif)
               r22 = dbrdss(nif)
               r12 = dbrdps(nif)
               r21 = dbrdsp(nif)
               rsh = dbrdsh(nif)
             else if ( cnvnif .eq. prmphs ) then
               r11 = dbrdpp(nif)
               r22 = dbrdss(nif)
               r12 = zero
               r21 = zero
               rsh = dbrdsh(nif)
             else if ( cnvnif .eq. cnvphs ) then
               r12 = dbrdps(nif)
               r21 = dbrdsp(nif)
               r11 = zero
               r22 = zero
               rsh = dbrdsh(nif)
             endif
            rnaupp = srnupp_bl(lyr)
            rnaups = srnups_bl(lyr)
            rnausp = srnusp_bl(lyr)
            rnauss = srnuss_bl(lyr)
            rnaush = srnush_bl(lyr)
            tndpp = stndpp_bl(lyr)
            tndps = stndps_bl(lyr)
            tndsp = stndsp_bl(lyr)
            tndss = stndss_bl(lyr)
            tndsh = stndsh_bl(lyr)
            rnadpp = srndpp_bltd(lyr)
            rnadps = srndps_bltd(lyr)
            rnadsp = srndsp_bltd(lyr)
            rnadss = srndss_bltd(lyr)
            rnadsh = srndsh_bltd(lyr)
c
c          form (I - Ru(lyr)*Rd(nif)]**-1 for downward tranmission
c          and reflection matrices Td, Rd (6.4, 6.18)
c
            if ( reverb(lyr) .eq. allrvb ) then
               l11 = one - (rnaupp*r11 + rnaups*r21)
               l22 = one - (rnausp*r12 + rnauss*r22)
               l12 = - (rnaupp*r12 + rnaups*r22)
               l21 = - (rnausp*r11 + rnauss*r21)
               det = ( l11*l22 - l12*l21 )
c            find inverse
               det = one / det
               l12 = -l12*det
               l21 = -l21*det
               t11 = l11*det
               l11 = l22*det
               l22 = t11
               lsh = one / ( one - rnaush*rsh )
             else if ( reverb(lyr) .eq. onervb ) then
               l11 = one + (rnaupp*r11 + rnaups*r21)
               l22 = one + (rnausp*r12 + rnauss*r22)
               l12 =  (rnaupp*r12 + rnaups*r22)
               l21 =  (rnausp*r11 + rnauss*r21)
               lsh = one + rnaush*rsh
             else if ( reverb(lyr) .eq. norvb ) then
               l11 = one
               l22 = one
               l12 = zero
               l21 = zero
               lsh = one
            endif
c
c        now finish the recursion, adding the next interface
c
            if ( cnvnif .eq. allphs ) then
               x11 = dbtdpp(nif)
               x22 = dbtdss(nif)
               x12 = dbtdps(nif)
               x21 = dbtdsp(nif)
               xsh = dbtdsh(nif)
               y11 = dbrupp(nif)
               y22 = dbruss(nif)
               y12 = dbrups(nif)
               y21 = dbrusp(nif)
               ysh = dbrush(nif)
             else if ( cnvnif .eq. prmphs ) then
               x11 = dbtdpp(nif)
               x22 = dbtdss(nif)
               x12 = zero
               x21 = zero
               xsh = dbtdsh(nif)
               y11 = dbrupp(nif)
               y22 = dbruss(nif)
               y12 = zero
               y21 = zero
               ysh = dbrush(nif)
             else if ( cnvnif .eq. cnvphs ) then
               x12 = dbtdps(nif)
               x21 = dbtdsp(nif)
               x11 = zero
               x22 = zero
               xsh = dbtdsh(nif)
               y12 = dbrups(nif)
               y21 = dbrusp(nif)
               y11 = zero
               y22 = zero
               ysh = dbrush(nif)
            endif
c
c         post multiply reverberation operator for Td (6.4)
c
            t11 = l11*tndpp + l12*tndsp
            t22 = l21*tndps + l22*tndss
            t21 = l21*tndpp + l22*tndsp
            t12 = l11*tndps + l12*tndss
            tsh = lsh * tndsh
c
c                               -1                    T
c        Rd = Rd + TuRd[I - RuRd] Td  ; but use Td = Tu  relation
c
            s11 = r11*t11 + r12*t21
            s12 = r11*t12 + r12*t22
            s21 = r21*t11 + r22*t21
            s22 = r21*t12 + r22*t22
            ssh = rsh*tsh
            rnadpp = rnadpp + tndpp*s11 + tndsp*s21
            rnadps = rnadps + tndpp*s12 + tndsp*s22
            rnadsp = rnadsp + tndps*s11 + tndss*s21
            rnadss = rnadss + tndps*s12 + tndss*s22
            rnadsh = rnadsh + tndsh*ssh
            if ( plyr .eq. btlyr ) then
               rpp = rnadpp
               rps = rnadps
               rsp = rnadsp
               rss = rnadss
               rsh = rnadsh
               return
            endif
c
c        tndpp = tdpp(nif)*t11 + tdps(nif)*t21
c        tndss = tdsp(nif)*t12 + tdss(nif)*t22
c        tndps = tdpp(nif)*t12 + tdps(nif)*t22
c        tndsp = tdsp(nif)*t11 + tdss(nif)*t21
c
c        pre mdltiply for Td (6.4, 6.18)
c        Td(z-) = Td(AB)[I - Ru(BC)Rd(AB)]**-1 Td(BC)
c
            tndpp = x11*t11 + x12*t21
            tndss = x21*t12 + x22*t22
            tndps = x11*t12 + x12*t22
            tndsp = x21*t11 + x22*t21
            tndsh = xsh * tsh
c
c       Now build Ru so the sense of the reverb operator becomes transposed since
c       the energy flow is initially upward for Ru, and initially downward for the
c       operators ( Rd and Td ) constructed above
c
c                   -1 T                -1
c       [ I - RuRd ]     =  [ I - RdRu ]
c
c       also using the transpose rule for
c              T
c       Tu = Td
c
c                                            -1
c       to build Ru = Ru + Td*Rd*[ I - Rd*Ru ] Tu  adding layers recursing down
c
c                 T            T    T                 T
c        tij = lij * Tu  =  lij * Td   =  [ Td * lij ]
c        t11 = l11*tdpp(nif) + l21*tdsp(nif)
c        t12 = l11*tdps(nif) + l21*tdss(nif)
c        t21 = l12*tdpp(nif) + l22*tdsp(nif)
c        t22 = l12*tdps(nif) + l22*tdss(nif)
c
            t11 = l11*x11 + l21*x12
            t12 = l11*x21 + l21*x22
            t21 = l12*x11 + l22*x12
            t22 = l12*x21 + l22*x22
            tsh = lsh * xsh
            l11 = rnaupp*t11 + rnaups*t21
            l12 = rnaupp*t12 + rnaups*t22
            l21 = rnausp*t11 + rnauss*t21
            l22 = rnausp*t12 + rnauss*t22
            lsh = rnaush * tsh
c
c        rnupp = rupp(nif) + tdpp(nif)*l11 + tdps(nif)*l21
c        rnuss = russ(nif) + tdsp(nif)*l12 + tdss(nif)*l22
c        rnups = rups(nif) + tdpp(nif)*l12 + tdps(nif)*l22
c        rnusp = rusp(nif) + tdsp(nif)*l11 + tdss(nif)*l21
c         Ru(AC) (6.3)
c
            rnaupp = y11 + x11*l11 + x12*l21
            rnauss = y22 + x21*l12 + x22*l22
            rnaups = y12 + x11*l12 + x12*l22
            rnausp = y21 + x21*l11 + x22*l21
            rnaush = ysh + xsh*lsh
c
	 endif
	 endif
c
c     Now structure is complete from top layer (tplyr) down to top of Perturbed Layer (plyr)
c
c============================================================================================
c
c  2. Now compute response for Region below Perturbed Layer (plyr)
c       now add perutbed interface (BotUp) to the region Below plyr
c       if plyr == tplyr   :  add perturbed interface and return
c       if tplyr + 1 > plyr > btlyr - 1 : add perturbed interface
c       if plyr == btlyr - 1  : just get the perturbed interface
c
      if ( plyr .lt. btlyr ) then
	 if ( plyr .eq. btlyr - 1 ) then
	    rnbdpp = dardpp(btlyr-1)
	    rnbdps = dardps(btlyr-1)
	    rnbdsp = dardsp(btlyr-1)
	    rnbdss = dardss(btlyr-1)
	    rnbdsh = dardsh(btlyr-1)
	  else
	    lyr = plyr + 1
            nif = plyr
	    rnbdpp = srndpp_blbu(lyr)
	    rnbdps = srndps_blbu(lyr)
	    rnbdsp = srndsp_blbu(lyr)
	    rnbdss = srndss_blbu(lyr)
	    rnbdsh = srndsh_blbu(lyr)
c
c        form the reverberation operator for the layer
c
            cnvnif = cnvrsn(nif)
            if ( cnvnif .eq. allphs ) then
               t11 = darupp(nif)
               t22 = daruss(nif)
               t12 = darups(nif)
               t21 = darusp(nif)
               tsh = darush(nif)
             else if ( cnvnif .eq. prmphs ) then
               t11 = darupp(nif)
               t22 = daruss(nif)
               t12 = zero
               t21 = zero
               tsh = darush(nif)
             else if ( cnvnif .eq. cnvphs ) then
               t12 = darups(nif)
               t21 = darusp(nif)
               t11 = zero
               t22 = zero
               tsh = darush(nif)
             endif
c
c         form [I - Rd Ru]**-1
c
            if ( reverb(lyr) .eq. allrvb ) then
               l11 = one - (rnbdpp*t11 + rnbdps*t21)
               l22 = one - (rnbdsp*t12 + rnbdss*t22)
               l12 = - (rnbdpp*t12 + rnbdps*t22)
               l21 = - (rnbdsp*t11 + rnbdss*t21)
               det = ( l11*l22 - l12*l21 )
               det = one / det
               l12 = -l12*det
               l21 = -l21*det
               t11 = l11*det
               l11 = l22*det
               l22 = t11
               lsh = one / ( one - rnbdsh*tsh )
             else if ( reverb(lyr) .eq. onervb ) then
               l11 = one + (rnbdpp*t11 + rnbdps*t21)
               l22 = one + (rnbdsp*t12 + rnbdss*t22)
               l12 =  (rnbdpp*t12 + rnbdps*t22)
               l21 =  (rnbdsp*t11 + rnbdss*t21)
               lsh = one + rnbdsh*tsh
             else if ( reverb(lyr) .eq. norvb ) then
               l11 = one
               l22 = one
               l12 = zero
               l21 = zero
               lsh = one
            endif
c
c        now finish the recursion (6.17), adding the next interface
c
            if ( cnvnif .eq. allphs ) then
               x11 = datupp(nif)
               x22 = datuss(nif)
               x12 = datups(nif)
               x21 = datusp(nif)
               xsh = datush(nif)
               y11 = dardpp(nif)
               y22 = dardss(nif)
               y12 = dardps(nif)
               y21 = dardsp(nif)
               ysh = dardsh(nif)
             else if ( cnvnif .eq. prmphs ) then
               x11 = datupp(nif)
               x22 = datuss(nif)
               x12 = zero
               x21 = zero
               xsh = datush(nif)
               y11 = dardpp(nif)
               y22 = dardss(nif)
               y12 = zero
               y21 = zero
               ysh = dardsh(nif)
             else if ( cnvnif .eq. cnvphs ) then
               x12 = datups(nif)
               x21 = datusp(nif)
               x11 = zero
               x22 = zero
               xsh = datush(nif)
               y12 = dardps(nif)
               y21 = dardsp(nif)
               y11 = zero
               y22 = zero
               ysh = dardsh(nif)
            endif
c
c        t11 = l11*tdpp(nif) + l21*tdsp(nif)
c        t12 = l11*tdps(nif) + l21*tdss(nif)
c        t21 = l12*tdpp(nif) + l22*tdsp(nif)
c        t22 = l12*tdps(nif) + l22*tdss(nif)
c
            t11 = l11*x11 + l21*x12
            t12 = l11*x21 + l21*x22
            t21 = l12*x11 + l22*x12
            t22 = l12*x21 + l22*x22
            tsh = lsh * xsh
            l11 = rnbdpp*t11 + rnbdps*t21
            l12 = rnbdpp*t12 + rnbdps*t22
            l21 = rnbdsp*t11 + rnbdss*t21
            l22 = rnbdsp*t12 + rnbdss*t22
            lsh = rnbdsh * tsh
c
c        rndpp = rdpp(nif) + tupp(nif)*l11 + tups(nif)*l21
c        rndss = rdss(nif) + tusp(nif)*l12 + tuss(nif)*l22
c        rndps = rdps(nif) + tupp(nif)*l12 + tups(nif)*l22
c        rndsp = rdsp(nif) + tusp(nif)*l11 + tuss(nif)*l21
c
            rnbdpp = y11 + x11*l11 + x12*l21
            rnbdss = y22 + x21*l12 + x22*l22
            rnbdps = y12 + x11*l12 + x12*l22
            rnbdsp = y21 + x21*l11 + x22*l21
            rnbdsh = ysh + xsh*lsh
c
            if ( plyr .eq. tplyr ) then
               rpp = rnbdpp
               rps = rnbdps
               rsp = rnbdsp
               rss = rnbdss
               rsh = rnbdsh
               return
            endif
	 endif
      endif
c
c================================================================================
c
c     3. Now Join the Results from above plyr and below plyr with the
c          Revererations in plyr using the phase from pertrubed vertical slowness.
c
      phtp = cphs( -i*w*dxi(plyr)*thik(plyr) )
      phts = cphs( -i*w*deta(plyr)*thik(plyr) )
      phtpp = phtp * phtp
      phtps = phtp * phts
      phtss = phts * phts
c
c         mult reflection coeff by corresponding phase delay using
c         recursive construction scheme (6.16) needed to transmit
c         up and down from reflection
c
      rnaupp = rnaupp * phtpp
      rnauss = rnauss * phtss
      rnaups = rnaups * phtps
      rnausp = rnausp * phtps
      rnaush = rnaush * phtss
c
c         for transmission, only need to transmit once
c
      tndpp = tndpp * phtp
      tndss = tndss * phts
      tndps = tndps * phtp
      tndsp = tndsp * phts
      tndsh = tndsh * phts
c
c          form (I - Ru*Rd]**-1 for downward
c          reflection matrix  Rd (6.4, 6.18)
c
      if ( reverb(lyr) .eq. allrvb ) then
         l11 = one - (rnaupp*rnbdpp + rnaups*rnbdsp)
         l22 = one - (rnausp*rnbdps + rnauss*rnbdss)
         l12 = - (rnaupp*rnbdps + rnaups*rnbdss)
         l21 = - (rnausp*rnbdpp + rnauss*rnbdsp)
         det = ( l11*l22 - l12*l21 )
c            find inverse
         det = one / det
         l12 = -l12*det
         l21 = -l21*det
         t11 = l11*det
         l11 = l22*det
         l22 = t11
         lsh = one / ( one - rnaush*rnbdsh )
       else if ( reverb(lyr) .eq. onervb ) then
         l11 = one + (rnaupp*rnbdpp + rnaups*rnbdsp)
         l22 = one + (rnausp*rnbdps + rnauss*rnbdss)
         l12 =  (rnaupp*rnbdps + rnaups*rnbdss)
         l21 =  (rnausp*rnbdpp + rnauss*rnbdsp)
         lsh = one + rnaush*rnbdsh
       else if ( reverb(lyr) .eq. norvb ) then
         l11 = one
         l22 = one
         l12 = zero
         l21 = zero
         lsh = one
      endif
c
c         post multiply reverberation operator for Td (6.4)
c
      t11 = l11*tndpp + l12*tndsp
      t22 = l21*tndps + l22*tndss
      t21 = l21*tndpp + l22*tndsp
      t12 = l11*tndps + l12*tndss
      tsh = lsh * tndsh
c
c                               -1                    T
c        Rd = Rd + TuRd[I - RuRd] Td  ; but use Td = Tu  relation
c
      s11 = rnbdpp*t11 + rnbdps*t21
      s12 = rnbdpp*t12 + rnbdps*t22
      s21 = rnbdsp*t11 + rnbdss*t21
      s22 = rnbdsp*t12 + rnbdss*t22
      ssh = rnbdsh*tsh
      rpp = rnadpp + tndpp*s11 + tndsp*s21
      rps = rnadps + tndpp*s12 + tndsp*s22
      rsp = rnadsp + tndps*s11 + tndss*s21
      rss = rnadss + tndps*s12 + tndss*s22
      rsh = rnadsh + tndsh*ssh
c
c
c
      return
      end
