      subroutine dispcc(p,f,islyr,d,nd,irlyr,recdep,nlyrs)
      implicit none
         complex p,f
         integer islyr,nlyrs,irlyr,nd
         real d(nd),recdep

c
c     compute the displacement transfer function of the medium
c     to upgoing and down going waves for a given
c     slowness p and frequency f
c     source is in layer islyr, and depth d from top of layer
c     model has total nlyrs including half space below
c
      include 'kennet.inc'
      complex rfspp,rfsps,rfssp,rfsss,rfssh
      complex rslpp,rslps,rslsp,rslss,rslsh
      complex tslpp,tslps,tslsp,tslss,tslsh
      complex t11,t12,t21,t22,tsh,l11,l12,l21,l22,lsh
      complex xpp,xps,xsp,xss,xsh,ypp,yps,ysp,yss,ysh
      complex rdvpup,rdvsup,rdrpup,rdrsup,rdtsup
      complex murvp, murrp, murvs, murrs
      complex mdrvp, mdrrp, mdrvs, mdrrs
      complex sphp, sphs, rphp, rphs
      complex srphpp, srphps, srphsp, srphss
c     complex*16 det
      complex det
      complex i,one,w,twopi,php,phs,phpp,phps,phss,oo,to,zero
      integer id, ii
      real rdepth, sdepth, ddepth, depth
      complex cphs
      external cphs
      data i,one,twopi/(0.,1.),(1.,0.),(6.2831853,0.)/
      data oo,to/(1.,1.),(2.,1.)/
      data zero/( 0.0, 0.0 )/
      w = twopi * f
c
c      Compute Displacement response for Code Coupling Studies
c      assuming receiver is in the half space (bottom layer)
c      Case 1 - source is also in half space and direct source
c      radiation is either up or down depending on source above
c      below the receiver, and reflected radiation from up
c      source radiation reflected from the surface layers
c      Case 2 - source is in the layers above the half space
c      and downward radiation is computed using the paper by
c      Marson-Pidgeon and Kennett, 2000 GJI Vol 143 pp 689-699
c
c
c      Displacement matrix m[ud]r[vr][ps]
c      for u,d up or downward radiation
c      for v,r vertical or radial displacement
c      for p,s p or s radiation
c      displacement matrix for receiver layer
c
       murvp = mu11(irlyr)
       murrp = mu21(irlyr)
       murvs = mu12(irlyr)
       murrs = mu22(irlyr)
c      mussh = mush(irlyr)
       mdrvp = md11(irlyr)
       mdrrp = md21(irlyr)
       mdrvs = md12(irlyr)
       mdrrs = md22(irlyr)
c      mdssh = mdsh(irlyr)
c      write(6,*) 'murvp, murrp, murvs, murrs ', murvp, murrp, murvs, murrs
c      write(6,*) 'mdrvp, mdrrp, mdrvs, mdrrs ', mdrvp, mdrrp, mdrvs, mdrrs
c
c====================================================================================
c
c      Case 1 - receiver is in the half space (bottom layer)
c      source is also in half space
c      depths are with respect to top of half space as calculated
c      in the mail program calling this routine
c      Phase terms for source up to layered bottom and down to receiver
c      with phase term for direct source to receiver up or down
c
       if ( ( irlyr .eq. nlyrs ) .and. ( islyr .eq. nlyrs ) ) then
           call rufs(p,f,islyr,rfspp,rfsps,rfssp,rfsss,rfssh)
           do 10 ii = 1,nd
               sdepth = d(ii)
               rdepth = recdep
               ddepth = abs( sdepth - rdepth )
               sphp = cphs( -i*w*xi(islyr)*sdepth )
               sphs = cphs( -i*w*eta(islyr)*sdepth )
               rphp = cphs( -i*w*xi(islyr)*rdepth )
               rphs = cphs( -i*w*eta(islyr)*rdepth )
               srphpp = sphp * rphp
               srphss = sphs * rphs
               srphsp = sphp * rphs
               srphps = sphs * rphp
c
c      relectivity from a plane wave incident from 
c      below, see Rufs in (7.36) upward P,S source radiation reflected back down
c
               xpp = rfspp * srphpp
               xps = rfsps * srphps
               xsp = rfssp * srphsp
               xss = rfsss * srphss
               xsh = rfssh * srphss
c
c      Direct Soruce radiation to receiver up (source below receiver)
c      or down (source above receiver) with no mode conversions
c
               sphp = cphs( -i*w*xi(islyr)*ddepth )
               sphs = cphs( -i*w*eta(islyr)*ddepth )
c
c      Combine Direct radiation (up or down) with initially upward
c      directed P,S reflected from layers bounded above by the free surface
c      Reflected same for both cases
c      For Receiver above Source (recdep < srcdep) add upbound direct, no downward direct
c      For Receiver below Source (recdep > srcdep) no upbound direct, add downward direct
c
               dvpup(ii) = mdrvp * xpp + mdrvs * xsp
               drpup(ii) = mdrrp * xpp + mdrrs * xsp
               dvsup(ii) = mdrvs * xss + mdrvp * xps
               drsup(ii) = mdrrs * xss + mdrrp * xps
               dtsup(ii) = mdssh * xsh
               if ( sdepth .gt. rdepth ) then
                   dvpup(ii) = murvp * sphp + dvpup(ii)
                   drpup(ii) = murrp * sphp + drpup(ii)
                   dvsup(ii) = murvs * sphs + dvsup(ii)
                   drsup(ii) = murrs * sphs + drsup(ii)
                   dtsup(ii) = mussh * sphs + dtsup(ii)
                   dvpdn(ii) = zero
                   drpdn(ii) = zero
                   dvsdn(ii) = zero
                   drsdn(ii) = zero
               else
                   dvpdn(ii) = mdrvp * sphp
                   drpdn(ii) = mdrrp * sphp
                   dvsdn(ii) = mdrvs * sphs
                   drsdn(ii) = mdrrs * sphs
               endif
10         continue
       return
       endif
c
c====================================================================================
c
c      Case 2 - receiver is in the half space (bottom layer)
c      source is above half space in layered structure
c      and downward radiation is computed using the paper by
c      Marson-Pidgeon and Kennett, 2000 GJI Vol 143 pp 689-699
c
       if ( ( irlyr .eq. nlyrs ) .and. ( islyr .lt. nlyrs ) ) then
           call rufs(p,f,islyr,rfspp,rfsps,rfssp,rfsss,rfssh)
           call refltrantd(p,f,islyr,nlyrs,rslpp,rslps,rslsp,rslss,rslsh,
     1                                     tslpp,tslps,tslsp,tslss,tslsh)
c          write(6,*) 'rfspp,rfsps,rfssp,rfsss,rfssh ',rfspp,rfsps,rfssp,rfsss,rfssh
c          write(6,*) 'rslpp,rslps,rslsp,rslss,rslsh ', rslpp,rslps,rslsp,rslss,rslsh
c          write(6,*) 'tslpp,tslps,tslsp,tslss,tslsh ', tslpp,tslps,tslsp,tslss,tslsh
           do 20 id = 1, nd
c
c     assemble the terms with appropriate phase delays for the required depths
c     for the Upward reflection delays
c
               depth = d(id)
               php = cphs( -i*w*xi(islyr)*depth )
               phs = cphs( -i*w*eta(islyr)*depth )
               phpp = php * php
               phss = phs * phs
               phps = php * phs
c
               rfspp = rfspp * phpp
               rfsps = rfsps * phps
               rfssp = rfssp * phps
               rfsss = rfsss * phss
               rfssh = rfssh * phss
c
c     assemble the terms with appropriate phase delays for the required depths
c     for the Downward refelection two way delays and 
c     Downward Transmission one way delay
c
               php = cphs( -i*w*xi(islyr)*(thik(islyr)-d(id)) )
               phs = cphs( -i*w*eta(islyr)*(thik(islyr)-d(id)) )
               phpp = php * php
               phss = phs * phs
               phps = php * phs
c
               rslpp = rslpp * phpp
               rslps = rslps * phps
               rslsp = rslsp * phps
               rslss = rslss * phss
               rslsh = rslsh * phss
               tslpp = tslpp * php
               tslps = tslps * php
               tslsp = tslsp * phs
               tslss = tslss * phs
               tslsh = tslsh * phs
c          write(6,*) 'rfspp,rfsps,rfssp,rfsss,rfssh ',rfspp,rfsps,rfssp,rfsss,rfssh
c          write(6,*) 'rslpp,rslps,rslsp,rslss,rslsh ', rslpp,rslps,rslsp,rslss,rslsh
c          write(6,*) 'tslpp,tslps,tslsp,tslss,tslsh ', tslpp,tslps,tslsp,tslss,tslsh

c
c     Form reverberation operator from rfs and rsl
c
c                   fS  SJ
c       form [I - Ru  Rd  ]**-1 in (2.8(s)) Marson-Pidgeon and Kennett
c
               l11 = -oo - rfspp * rslpp - rfsps * rslsp
               l12 = -oo - rfsps * rslss - rfspp * rslps
               l21 = -oo - rfssp * rslpp - rfssp * rslps
               l22 = -oo - rfsss * rslss - rfssp * rslps
               l11 = to + l11
               l12 = oo + l12
               l21 = oo + l21
               l22 = to + l22
               det = l11*l22 - l12*l21
               det = one / det
               l12 = -l12*det
               l21 = -l21*det
               t11 = l11*det
               l11 = l22*det
               l22 = t11
               lsh = one / ( one - rfssh*rslsh )
c
c     Form Downward Transmission
c
c              SJ        fS  SJ
c       form Td * [I - Ru  Rd  ]**-1 in (2.8(s)) Marson-Pidgeon and Kennett
c
               t11 = tslpp * l11 + tslps * l21
               t12 = tslps * l22 + tslpp * l12
               t21 = tslss * l21 + tslsp * l11
               t22 = tslss * l22 + tslsp * l12
               tsh = tslsh * lsh
c
c    Add one way travel time in half space down to receiver depth
c
               php = cphs( -i*w*xi(irlyr)*recdep )
               phs = cphs( -i*w*eta(irlyr)*recdep )
               tslpp = t11 * php
               tslps = t12 * php
               tslsp = t21 * phs
               tslss = t22 * phs
               tslsh = tsh * phs
c          write(6,*) 'tslpp,tslps,tslsp,tslss,tslsh ', tslpp,tslps,tslsp,tslss,tslsh

c
c    Form the Receiver Displacement for Downgoing waves
c
               dvpdn(id) = mdrvp * tslpp + mdrvs * tslsp
               drpdn(id) = mdrrp * tslpp + mdrrs * tslsp
               drsdn(id) = mdrrs * tslss + mdrrp * tslps
               dvsdn(id) = mdrvs * tslss + mdrvp * tslps
               dtsdn(id) = mdssh * tslsh
c          write(6,*) 'tslpp,tslps,tslsp,tslss,tslsh ', tslpp,tslps,tslsp,tslss,tslsh
c
c     Form Net Downward Transmission for Initial Upward Radiation
c
c              SJ        fS  SJ          fS
c       form Td * [I - Ru  Rd  ]**-1 * Ru  in (2.8(s)) Marson-Pidgeon and Kennett
c
               t11 = tslpp * rfspp + tslps * rfssp
               t12 = tslpp * rfsps + tslps * rfsss
               t21 = tslsp * rfspp + tslss * rfssp
               t22 = tslsp * rfsps + tslss * rfsss
               tsh = tslsh * rfssh
c          write(6,*) 'dvpdn(id),drpdn(id),drsdn(id),dvsdn(id),dtsdn(id) ', dvpdn(id),drpdn(id),drsdn(id),dvsdn(id),dtsdn(id)

c
c    Form the Receiver Displacement for Upgoing waves
c
               dvpup(id) = mdrvp * t11 + mdrvs * t21
               drpup(id) = mdrrp * t11 + mdrrs * t21
               drsup(id) = mdrrs * t22 + mdrrp * t12
               dvsup(id) = mdrvs * t22 + mdrvp * t12
               dtsup(id) = mdssh * tsh
               
c          write(6,*) 'dvpup(id),drpup(id),drsup(id),dvsup(id),dtsup(id) ', dvpup(id),drpup(id),drsup(id),dvsup(id),dtsup(id)
20         continue
           return
       endif
c
      write(6,*) 'If we get here, something is wrong!'
c
      return
      end
