      subroutine refltd(p,f,tplyr,btlyr,rpp,rps,rsp,rss,rsh)
      integer tplyr,btlyr
      complex p,f
      complex rpp,rps,rsp,rss,rsh
c
c        compute reflectivity - reflection from a
c        plane wave incident from above, on a stack of plane, parallel,
c        homogeneous layers
c        for a p, sv or sh wave incident
c        top interface is top of layer tplyr
c        layer btlyr is half space, with radiation condition
c        given frequency and phase slowness.
c
c          arguments...
c
c        f,p - prescribed freq (hz) & horizontal phase slowness (c is
c            not restricted to be greater than alfa or beta)
c            both may be complex
c
c        passed in common /model/
c        alfa,beta,qp,qs,rho and thik contain the medium properties for
c
c        nlyrs - total number of layers, layer nlyrs is
c            the half space
c
c
c
c        commons and declarations
c
c
      include 'kennet.inc'
c
c        complex declarations
c
      complex i,zero,one,w
      complex t11,t12,t21,t22,l11,l12,l21,l22,tsh,lsh
c     complex*16 det
      complex det
      complex x11,x12,x21,x22,y11,y12,y21,y22,xsh,ysh
      complex r11,r12,r21,r22,s11,s12,s21,s22,ssh
      complex tndpp,tndps,tndsp,tndss,tndsh
      complex rndpp,rndps,rndsp,rndss,rndsh
      complex rnupp,rnups,rnusp,rnuss,rnush
      complex phtp,phts,phtpp,phtps,phtss
      real twopi
      integer lyr,nif,cnvnif
      external cphs
      complex cphs
      data twopi/6.2831853/,i,zero/(0.,1.),(0.,0.)/,one/(1.,0.)/
c
c
      w = twopi*f
c
c
c        initialize tdown, rup and rdown matrices for the stack with
c        interface matrices
c        cnvrsn set in kennett.f
c        allphs - all converted phases at layer using trans/ref matrices
c                 in ifmat.f
c        prmphs - no converted phases, only PP, SS reflections
c        cnvphs - only converted phases
c
      nif = tplyr
      cnvnif = cnvrsn(nif)
      if ( cnvnif .eq. allphs ) then
         tndpp = tdpp(nif)
         tndss = tdss(nif)
         tndps = tdps(nif)
         tndsp = tdsp(nif)
         tndsh = tdsh(nif)
         rndpp = rdpp(nif)
         rndss = rdss(nif)
         rndps = rdps(nif)
         rndsp = rdsp(nif)
         rndsh = rdsh(nif)
         rnupp = rupp(nif)
         rnuss = russ(nif)
         rnups = rups(nif)
         rnusp = rusp(nif)
         rnush = rush(nif)
       else if ( cnvnif .eq. prmphs ) then
         tndpp = tdpp(nif)
         tndss = tdss(nif)
         tndps = zero
         tndsp = zero
         tndsh = tdsh(nif)
         rndpp = rdpp(nif)
         rndss = rdss(nif)
         rndps = zero
         rndsp = zero
         rndsh = rdsh(nif)
         rnupp = rupp(nif)
         rnuss = russ(nif)
         rnups = zero
         rnusp = zero
         rnush = rush(nif)
       else if ( cnvnif .eq. cnvphs ) then
         tndps = tdps(nif)
         tndsp = tdsp(nif)
         tndpp = zero
         tndss = zero
         tndsh = tdsh(nif)
         rndps = rdps(nif)
         rndsp = rdsp(nif)
         rndpp = zero
         rndss = zero
         rndsh = rdsh(nif)
         rnups = rups(nif)
         rnusp = rusp(nif)
         rnupp = zero
         rnuss = zero
         rnush = rush(nif)
       endif
c
c        now do the  bottom up recursion for tup and rdown
c
      do 10 lyr = tplyr+1, btlyr-1
c     nif is next layer above lyr
         nif = lyr
c
c        use the two way phase delay through the layer
c        to/from the next interface (3.11, 3.12)
c
         phtp = cphs( -i*w*xi(lyr)*thik(lyr) )
         phts = cphs( -i*w*eta(lyr)*thik(lyr) )
         phtpp = phtp * phtp
         phtps = phtp * phts
         phtss = phts * phts
c
c         mult reflection coeff by corresponding phase delay using
c         recursive construction scheme (6.16) needed to transmit
c         up and down from reflection
c
         rnupp = rnupp * phtpp
         rnuss = rnuss * phtss
         rnups = rnups * phtps
         rnusp = rnusp * phtps
         rnush = rnush * phtss
c
c         for transmission, only need to transmit once
c
         tndpp = tndpp * phtp
         tndss = tndss * phts
         tndps = tndps * phtp
         tndsp = tndsp * phts
         tndsh = tndsh * phts
#ifdef DIFFSEIS
c
c         store for each layer
c
	 stndpp_bl(lyr) = tndpp
	 stndps_bl(lyr) = tndps
	 stndsp_bl(lyr) = tndsp
	 stndss_bl(lyr) = tndss
	 stndsh_bl(lyr) = tndsh
	 srndpp_bltd(lyr) = rndpp
	 srndps_bltd(lyr) = rndps
	 srndsp_bltd(lyr) = rndsp
	 srndss_bltd(lyr) = rndss
	 srndsh_bltd(lyr) = rndsh
	 srnupp_bl(lyr) = rnupp
	 srnups_bl(lyr) = rnups
	 srnusp_bl(lyr) = rnusp
	 srnuss_bl(lyr) = rnuss
	 srnush_bl(lyr) = rnush
#endif
c
c        form the reverberation operator for the layer
c        nif is below above lyr, so tii is Rd(nif)
c
         cnvnif = cnvrsn(nif)
         if ( cnvnif .eq. allphs ) then
            r11 = rdpp(nif)
            r22 = rdss(nif)
            r12 = rdps(nif)
            r21 = rdsp(nif)
            rsh = rdsh(nif)
          else if ( cnvnif .eq. prmphs ) then
            r11 = rdpp(nif)
            r22 = rdss(nif)
            r12 = zero
            r21 = zero
            rsh = rdsh(nif)
          else if ( cnvnif .eq. cnvphs ) then
            r12 = rdps(nif)
            r21 = rdsp(nif)
            r11 = zero
            r22 = zero
            rsh = rdsh(nif)
          endif
c
c          form (I - Ru(lyr)*Rd(nif)]**-1 for downward tranmission
c          and reflection matrices Td, Rd (6.4, 6.18)
c
         if ( reverb(lyr) .eq. allrvb ) then
            l11 = one - (rnupp*r11 + rnups*r21)
            l22 = one - (rnusp*r12 + rnuss*r22)
            l12 = - (rnupp*r12 + rnups*r22)
            l21 = - (rnusp*r11 + rnuss*r21)
            det = ( l11*l22 - l12*l21 )
c            find inverse
	    det = one / det
            l12 = -l12*det
            l21 = -l21*det
            t11 = l11*det
            l11 = l22*det
            l22 = t11
            lsh = one / ( one - rnush*rsh )
         else if ( reverb(lyr) .eq. onervb ) then
            l11 = one + (rnupp*r11 + rnups*r21)
            l22 = one + (rnusp*r12 + rnuss*r22)
            l12 =  (rnupp*r12 + rnups*r22)
            l21 =  (rnusp*r11 + rnuss*r21)
            lsh = one + rnush*rsh
         else if ( reverb(lyr) .eq. norvb ) then
            l11 = one
            l22 = one
            l12 = zero
            l21 = zero
            lsh = one
          endif
c
c        now finish the recursion, adding the next interface
c
         if ( cnvnif .eq. allphs ) then
            x11 = tdpp(nif)
            x22 = tdss(nif)
            x12 = tdps(nif)
            x21 = tdsp(nif)
            xsh = tdsh(nif)
            y11 = rupp(nif)
            y22 = russ(nif)
            y12 = rups(nif)
            y21 = rusp(nif)
            ysh = rush(nif)
          else if ( cnvnif .eq. prmphs ) then
            x11 = tdpp(nif)
            x22 = tdss(nif)
            x12 = zero
            x21 = zero
            xsh = tdsh(nif)
            y11 = rupp(nif)
            y22 = russ(nif)
            y12 = zero
            y21 = zero
            ysh = rush(nif)
          else if ( cnvnif .eq. cnvphs ) then
            x12 = tdps(nif)
            x21 = tdsp(nif)
            x11 = zero
            x22 = zero
            xsh = tdsh(nif)
            y12 = rups(nif)
            y21 = rusp(nif)
            y11 = zero
            y22 = zero
            ysh = rush(nif)
          endif
c
c         post multiply reverberation operator for Td (6.4)
c
         t11 = l11*tndpp + l12*tndsp
         t22 = l21*tndps + l22*tndss
         t21 = l21*tndpp + l22*tndsp
         t12 = l11*tndps + l12*tndss
         tsh = lsh * tndsh
c
c                               -1                    T
c        Rd = Rd + TuRd[I - RuRd] Td  ; but use Td = Tu  relation
c
	 s11 = r11*t11 + r12*t21
	 s12 = r11*t12 + r12*t22
	 s21 = r21*t11 + r22*t21
	 s22 = r21*t12 + r22*t22
	 ssh = rsh*tsh
	 rndpp = rndpp + tndpp*s11 + tndsp*s21
	 rndps = rndps + tndpp*s12 + tndsp*s22
	 rndsp = rndsp + tndps*s11 + tndss*s21
	 rndss = rndss + tndps*s12 + tndss*s22
	 rndsh = rndsh + tndsh*ssh
c
c        tndpp = tdpp(nif)*t11 + tdps(nif)*t21
c        tndss = tdsp(nif)*t12 + tdss(nif)*t22
c        tndps = tdpp(nif)*t12 + tdps(nif)*t22
c        tndsp = tdsp(nif)*t11 + tdss(nif)*t21
c         pre mdltiply for Td (6.4, 6.18)
c         Td(z-) = Td(AB)[I - Ru(BC)Rd(AB)]**-1 Td(BC)
c
         tndpp = x11*t11 + x12*t21
         tndss = x21*t12 + x22*t22
         tndps = x11*t12 + x12*t22
         tndsp = x21*t11 + x22*t21
         tndsh = xsh * tsh
c
c       Now build Ru so the sense of the reverb operator becomes transposed since
c       the energy flow is initially upward for Ru, and initially downward for the
c       operators ( Rd and Td ) constructed above
c
c                   -1 T                -1
c       [ I - RuRd ]     =  [ I - RdRu ]
c
c       also using the transpose rule for
c              T
c       Tu = Td
c
c                                            -1
c       to build Ru = Ru + Td*Rd*[ I - Rd*Ru ] Tu  adding layers recursing down
c
c                 T            T    T                 T
c        tij = lij * Tu  =  lij * Td   =  [ Td * lij ]
c        t11 = l11*tdpp(nif) + l21*tdsp(nif)
c        t12 = l11*tdps(nif) + l21*tdss(nif)
c        t21 = l12*tdpp(nif) + l22*tdsp(nif)
c        t22 = l12*tdps(nif) + l22*tdss(nif)
         t11 = l11*x11 + l21*x12
         t12 = l11*x21 + l21*x22
         t21 = l12*x11 + l22*x12
         t22 = l12*x21 + l22*x22
         tsh = lsh * xsh
         l11 = rnupp*t11 + rnups*t21
         l12 = rnupp*t12 + rnups*t22
         l21 = rnusp*t11 + rnuss*t21
         l22 = rnusp*t12 + rnuss*t22
         lsh = rnush * tsh
c        rnupp = rupp(nif) + tdpp(nif)*l11 + tdps(nif)*l21
c        rnuss = russ(nif) + tdsp(nif)*l12 + tdss(nif)*l22
c        rnups = rups(nif) + tdpp(nif)*l12 + tdps(nif)*l22
c        rnusp = rusp(nif) + tdsp(nif)*l11 + tdss(nif)*l21
c         Ru(AC) (6.3)
         rnupp = y11 + x11*l11 + x12*l21
         rnuss = y22 + x21*l12 + x22*l22
         rnups = y12 + x11*l12 + x12*l22
         rnusp = y21 + x21*l11 + x22*l21
         rnush = ysh + xsh*lsh
c
10    continue
      rpp = rndpp
      rps = rndps
      rsp = rndsp
      rss = rndss
      rsh = rndsh
c
c
c
      return
      end
